libpth="$libpth /usr/shlib" # Use the shared libraries if possible
libc='/usr/shlib/libc.so'   # The archive version is /lib/libc.a
d_berknames='define'        # Digital/Tru64 does use Berkeley format GCOS
