/* filter.h -- declarations for filter.c.
 */
/* This software is copyrighted as detailed in the LICENSE file. */


/* DON'T EDIT BELOW THIS LINE OR YOUR CHANGES WILL BE LOST! */

void filter_init _((void));
void filter_nginit _((void));
int filter _((ART_NUM));
void filter_cleanup _((void));
