/* autosub.h
 */
/* This software is copyrighted as detailed in the LICENSE file. */


/* DON'T EDIT BELOW THIS LINE OR YOUR CHANGES WILL BE LOST! */

int auto_subscribe _((char*));
bool matchlist _((char*,char*));
