#! /bin/sh
#
# This file was hand-generated, since Configure doesn't run under msdos. It
# holds all the definitions for configuring trn.  You should only modify
# the values commented at the start of this file.  To make use of this
# file you need to have a bourne shell installed (/bin/sh) and various
# un*x utilities, such as sed and awk.  You also need a un*x compatible
# make utility that understands VPATH, such as dmake.
#

srcdir='..'
vincludes='-I..'
vpath='..'
cc='bcc'
ccflags='-WX -ml -dc -w-par -w-stu -w-pia -DDEBUG'
cppflags=''
ldflags='-v -WX'
lkflags='/c -v -Txe -Lc:\bc4\lib'
optimize='-v'
cpplast=''
cppminus=''
cpprun=''
cppstdin='cppstdin'
prefix='c:/trn'
bin='~~/bin'
privlib='~~/lib'
newslib='c:/tmp'
usrinc='\bc4\include'

sharpbang='#! '
startsh='#! /bin/sh'
orgname='Total lack thereof'
inewsloc='inews'
pager='more'
defeditor='edit'
ispell_options='-x'
ispell_prg='ispell'
mailer='mail'
mailfile='%~/%L'
phost='.'
phostcmd='hostname'
prefshell='command.com'
trn_init='FALSE'
trn_select='TRUE'

# The default distributions

orgdist='none'
locdist='local'
citydist='ba'
statedist='ca'
multistatedist='none'
cntrydist='usa'
contdist='na'

d_genauth='undef'
d_nntp='undef'
d_xthread="$d_nntp"

#
# You could alternately use the access.default file in the trn library to
# change the default data source for all users.
#

servername='local'
active='/usr/lib/news/active'
newsspool='/usr/spool/news'
d_acttimes='define'
acttimes="$active.times"
overviewdir='remote'
threaddir='remote'

#
# You shouldn't need to change any of these.
#

cf_by='<unknown>'
cf_time='<unknown>'
myuname='<unknown>'
osname='MSDOS'
osvers='<unknown>'

filexp='Rnfilexp'
direntrytype='struct dirent'
i_dirent='define'
i_stdlib='define'
i_string='define'
strings="$usrinc/string.h"

c='-c'
n=''
d_bsd='undef'
d_eunice='undef'
d_xenix='undef'
eunicefix=':'
Mcc='Mcc'
awk='awk'
basename='basename'
bash='bash'
byacc='byacc'
cat='cat'
cp='cp'
cpp='cpp'
csh='csh'
date='date'
diff='diff'
echo='echo'
ed='ed'
egrep='egrep'
expr='expr'
find='find'
grep='grep'
inews='inews'
ispell='ispell'
ksh='ksh'
less='less'
lint='lint'
mail='mail'
metamail='metamail'
mhn='mhn'
mkdir='mkdir'
more='more'
munpack='munpack'
mv='mv'
nroff='nroff'
pg='pg'
rm='rm'
rmail='rmail'
sed='sed'
sendmail='sendmail'
smail='smail'
sort='sort'
spell='spell'
tail='tail'
test='test'
tr='tr'
uname='uname'
uniq='uniq'
uuname='uuname'
vi='vi'
vspell='vspell'
who='who'
hint='hint'
Id='Id'
Log='Log'
myactive="$active"
binexp="$bin"
installbin="$bin"
gccversion=''
contains="$grep"
d_attribut='undef'
d_ftime='define'
aphostcmd='hostname'
d_getdname='undef'
d_gethname='undef'
d_phostcmd='undef'
d_resinit='undef'
d_uname='undef'
d_getpwent='undef'
d_getcwd='define'
d_getwd='undef'
d_havetlib='define'
termlib=''
d_ignoreorg='undef'
d_inews='define'
d_internet='define'
d_memcmp='define'
d_memcpy='define'
d_memset='define'
d_mimeshow='undef'
d_mimestore='undef'
d_msdos='define'
mimeshow="none"
mimestore="none"
d_mkdir='define'
d_newsadm='undef'
newsadmin='root'
d_nolnbuf='undef'
d_normsig='define'
jobslib=''
d_novoid='undef'
void=''
d_portable='define'
d_rdchk='undef'
d_rename='define'
d_sigblock='undef'
d_sighold='undef'
d_sizet='undef'
d_strccmp='undef'
d_strchr='define'
d_strftime='define'
strftimec=''
strftimeo=''
d_libndir='undef'
d_usendir='undef'
libndir=''
ndirc=''
ndiro=''
d_vfork='undef'
usevfork='undef'
d_voidsig='define'
signal_t='void'
d_dirnamlen='undef'
i_ptem='undef'
i_sysdir='undef'
d_voidtty='undef'
i_bsdioctl='undef'
i_sysfilio='undef'
i_sysioctl='undef'
i_syssockio='undef'
i_sysndir='undef'
i_sgtty='undef'
i_termio='undef'
i_termios='undef'
i_systime='undef'
i_time='define'
i_unistd='undef'
i_vfork='undef'
libc=''
glibpth=''
plibpth=''
xlibpth=''
libs=''
installmansrc=' '
manext='1'
mansrc=' '
mansrcexp=' '
mboxchar='F'
d_berknames='undef'
d_passnames='undef'
d_usgnames='undef'
nametype='other'
newslibexp="$newslib"
package='trn'
spackage='Trn'
installprivlib="$privlib"
privlibexp="$privlib"
rootid='0'
so='so'
shsharp='true'
spitshell="$cat"
sysman='/usr/man/man1'
nm_opt=''
runnm='false'
usenm='false'
incpath=''
mips=''
mips_type=''
