/* This file Copyright 1992 by Clifford A. Adams */
/* score-easy.h
 *
 */

/* DON'T EDIT BELOW THIS LINE OR YOUR CHANGES WILL BE LOST! */

char* sc_easy_append _((void));
char* sc_easy_command _((void));
