/* This file Copyright 1992 by Clifford A. Adams */
/* sadisp.h
 *
 * article scan specific display functions
 */

/* DON'T EDIT BELOW THIS LINE OR YOUR CHANGES WILL BE LOST! */

void sa_refresh_top _((void));
void sa_refresh_bot _((void));
void sa_set_screen _((void));
