/* tktree.h
 */

/*
 * The authors make no claims as to the fitness or correctness of this software
 * for any use whatsoever, and it is provided as is. Any use of this software
 * is at the user's own risk. 
 */

/* DON'T EDIT BELOW THIS LINE OR YOUR CHANGES WILL BE LOST! */

void ttk_draw_tree _((ARTICLE*,int,int));
int ttk_tree_init _((void));
